#!/usr/bin/env python
"""
exceptions.py
"""

import logging

from django.core.paginator import EmptyPage
from django.db import DatabaseError
from django.http import Http404
from django.utils.translation import gettext_lazy as _
from rest_framework import status, exceptions
from rest_framework.exceptions import (
    APIException,
    ValidationError,
    AuthenticationFailed,
    PermissionDenied,
    NotAuthenticated,
    MethodNotAllowed,
    NotFound,
)
from rest_framework.views import exception_handler, set_rollback

from drfcommon.choices import ComCodeChoice
from drfcommon.log import logger_print_by_code
from drfcommon.response import done


class ComValidationError(ValidationError):
    """
    ComValidation Error
    """
    status_code = status.HTTP_400_BAD_REQUEST

    def __init__(self, detail=None, code=None):
        if code:
            self.status_code = code
        super().__init__(detail=detail, code=code)


class ComAPIException(APIException):
    """
    ComAPIException detail 只返回string
    """
    status_code = status.HTTP_200_OK
    default_detail = _('A server error occurred.')
    default_code = 'error'
    err_code = status.HTTP_200_OK

    def __init__(self, detail=None, err_code=None):
        if err_code:
            self.err_code = err_code
        super().__init__(detail=detail, code=self.status_code)


def exception_handler(exc, context):
    """
    Returns the response that should be used for any given exception.

    By default we handle the REST framework `APIException`, and also
    Django's built-in `Http404` and `PermissionDenied` exceptions.

    Any unhandled exceptions may return `None`, which will cause a 500 error
    to be raised.
    """
    code = ComCodeChoice.BAD
    msg = None
    if isinstance(exc, Http404):
        code = ComCodeChoice.API_NOT_FUND
    elif isinstance(exc, EmptyPage):
        # 404
        code = ComCodeChoice.API_NOT_FUND
    elif isinstance(exc, NotFound):
        # 404
        code = ComCodeChoice.API_NOT_FUND
    elif isinstance(exc, ValidationError):
        # 400
        code = ComCodeChoice.BAD
    elif isinstance(exc, ComValidationError):
        # 400
        code = ComCodeChoice.BAD
    elif isinstance(exc, NotAuthenticated):
        # 401
        code = ComCodeChoice.UNAUTHORIZED_ERR
    elif isinstance(exc, AuthenticationFailed):
        # 401
        code = ComCodeChoice.UNAUTHORIZED_ERR
    elif isinstance(exc, MethodNotAllowed):
        code = ComCodeChoice.HTTP_405_METHOD_NOT_ALLOWED
    elif isinstance(exc, PermissionDenied):
        # 403
        code = ComCodeChoice.FORBIDDEN_ERR
    elif isinstance(exc, DatabaseError):
        code = ComCodeChoice.DB_ERR
    else:
        # 如果没有处理，保留原始的错误
        msg = "{}".format(exc)
    logger_print_by_code(code, exc)
    # msg 是否被设置. 无，使用自定义
    if not msg:
        msg = ComCodeChoice.choices_map[code]
    if isinstance(exc, exceptions.APIException):
        headers = {}
        if getattr(exc, 'auth_header', None):
            headers['WWW-Authenticate'] = exc.auth_header
        if getattr(exc, 'wait', None):
            headers['Retry-After'] = '%d' % exc.wait

        if isinstance(exc.detail, (list, dict)):
            data = exc.detail
        else:
            data = {'detail': exc.detail}
        set_rollback()
        return done(
            code=code,
            msg=msg,
            # dict
            errors=data,
        )
    return done(code=code, msg=msg)


def com_exception_handler(exc, context):
    """
    处理views中的异常, 视图函数只返回200，errmsg/errcode

    exc.detail
        if isinstance(exc.detail, (list, dict)):
            data = exc.detail
        else:
            data = {'detail': exc.detail}

    :param exc: APIException
    :param context:
    :return:
    """
    response = exception_handler(exc, context)
    return response




#!/usr/bin/env python
import logging
from enum_choices import Choices


class ComCodeChoice(Choices):
    """
    ComCodeChoice
    """
    OK = (200, '成功')
    BAD = (400, '参数错误')
    UNAUTHORIZED_ERR = (401, "认证失败")
    FORBIDDEN_ERR = (403, '权限错误')
    API_NOT_FUND = (404, '找不到对应的服务或数据')
    HTTP_405_METHOD_NOT_ALLOWED = (405, '方法不允许')
    API_ERR = (500, '内部错误')
    DB_ERR = (507, '服务器内部数据库问题')




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
doc:
"""
from rest_framework.response import Response


def make_data(code=200, msg='', **kwargs):
    """

    :param msg:
    :param code: 200(成功), 101(参数错误), 199(其他)
    :return: dict
    """
    if "data" not in kwargs:
        kwargs["data"] = dict()

    if "lists" not in kwargs:
        kwargs["lists"] = list()
    resp = dict(
        code=code,
        msg=msg,
        **kwargs
    )
    return resp


def make_response(code=200, msg='成功', **kwargs):
    """

    :param msg:
    :param code: 200(成功), 101(参数错误), 199(其他),
    :return: Response
    """
    data = make_data(code=code, msg=msg, **kwargs)
    return Response(data)


def done(code=200, msg='成功', **kwargs):
    """

    :param code:
    :param msg:
    :param kwargs:
    :return: Response
    """
    data = make_response(
        code=code,
        msg=msg,
        **kwargs
    )
    return data



#!/usr/bin/env python
from collections import OrderedDict

from rest_framework.pagination import PageNumberPagination
from rest_framework.response import Response


class ComPagination(PageNumberPagination):
    """
    自定义分页器
    """
    page_size = 5
    page_size_query_param = 'pagesize'
    max_page_size = 50

    def get_paginated_response(self, data):
        return Response(OrderedDict([
            ('count', self.page.paginator.count),
            ('page', self.page.number),
            # 总页数量
            ('pages', self.page.paginator.num_pages),
            # 每页数量
            ('pagesize', self.get_page_size(self.request)),
            ('lists', data)
        ]))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
doc:
"""
import logging

from rest_framework.viewsets import ModelViewSet
from drfcommon.exceptions import com_exception_handler
from drfcommon.response import done


logger = logging.getLogger('debug')


class AllowAnyModelViewSet(ModelViewSet):
    """
    AllowAny  ModelViewSet
    """
    http_method_names = ['get', 'post', 'put', 'delete', 'options', ]
    serializer_map = dict()

    def get_serializer_class(self):
        if not isinstance(self.serializer_map, dict):
            return self.serializer_class
        if self.action not in self.serializer_map:
            logger.warning('action:{} not conf serializer'.format(self.action))
        return self.serializer_map.get(self.action, self.serializer_class)


class ComApiBaseModelSet(AllowAnyModelViewSet):
    """
    Com App Base ModelViewSet

    base method:
    1.get_exception_handler
    2.initialize_request
    3.errors
    4.done

    restful method:
    1.list:
    2.retrieve
    3.update
    4.create
    """

    def get_exception_handler(self):
        """
        Returns the exception handler that this view uses.
        """
        return com_exception_handler

    @staticmethod
    def errors(errors):
        logger.warning("errors:{}".format(errors))
        code = 400
        return done(
            code=code,
            msg='请检查请求参数',
            errors=errors,
        )

    def destroy(self, request, *args, **kwargs):
        """
        删除: 通过主键id

        ----

        """
        instance = self.get_object()
        self.perform_destroy(instance)
        return done()

    def perform_destroy(self, instance):
        """
        删除: 逻辑删除
        :param instance:
        :return:
        """
        instance.deleted = True
        instance.save()

    def list(self, request, *args, **kwargs):
        """
        列表
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        queryset = self.filter_queryset(self.get_queryset())
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)
        serializer = self.get_serializer(queryset, many=True)
        return done(data={}, lists=serializer.data)

    def retrieve(self, request, *args, **kwargs):
        """
        详情
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        instance = self.get_object()
        serializer = self.get_serializer(instance)
        return done(data=serializer.data)

    def update(self, request, *args, **kwargs):
        """
        更新
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=request.data,
                                         partial=partial)
        if not serializer.is_valid():
            return self.errors(serializer.errors)
        self.perform_update(serializer)
        if getattr(instance, '_prefetched_objects_cache', None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}
        return done(data=serializer.data)

    def perform_update(self, serializer):
        serializer.save()

    def partial_update(self, request, *args, **kwargs):
        kwargs['partial'] = True
        return self.update(request, *args, **kwargs)

    def create(self, request, *args, **kwargs):
        """
        创建
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        serializer = self.get_serializer(data=request.data)
        if not serializer.is_valid():
            return self.errors(serializer.errors)
        validated_data = serializer.validated_data
        # 创建逻辑: 由调用方序列化定制
        instance = serializer.create(validated_data)
        # data = validated_data
        data = dict()
        data['id'] = instance.id
        return done(data=data)

    def bulk_create(self, request, *args, **kwargs):
        """
        批量创建
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        serializer = self.get_serializer(data=request.data, many=True)
        if not serializer.is_valid():
            return self.errors(serializer.errors)
        validated_data = serializer.validated_data
        instance_list = serializer.create(validated_data)
        data = dict()
        data['id_list'] = [instance.id for instance in instance_list]
        return done(data=data)



#!/usr/bin/env python
import logging
from drfcommon.choices import ComCodeChoice

logger = logging.getLogger('debug')


LOG_PRINT_FUNC = {
    ComCodeChoice.OK: logger.debug,
    ComCodeChoice.BAD: logger.warning,
    ComCodeChoice.UNAUTHORIZED_ERR: logger.warning,
    ComCodeChoice.FORBIDDEN_ERR: logger.warning,
    ComCodeChoice.API_NOT_FUND: logger.warning,
    ComCodeChoice.HTTP_405_METHOD_NOT_ALLOWED: logger.warning,
    ComCodeChoice.API_ERR: logger.error,
    ComCodeChoice.DB_ERR: logger.error,
}


def logger_print_by_code(code: ComCodeChoice, exc: Exception):
    """

    :param code: ComCodeChoice
    :param exc:
    :return:
    """
    logger_print = LOG_PRINT_FUNC.get(code, logger.error)
    exc_info = False
    if logger_print == logger.error:
        exc_info = True
    logger_print("raw exc {}".format(exc), exc_info=exc_info)
    return



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['drfcommon']

package_data = \
{'': ['*']}

install_requires = \
['django>=2.2', 'djangorestframework>=3.10']

setup_kwargs = {
    'name': 'drfcommon',
    'version': '0.1.9.5',
    'description': 'a common tools for drf',
    'long_description': 'None',
    'author': 'tplinux',
    'author_email': '2363176358@qq.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/pydtools/drfcommon/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8',
}


setup(**setup_kwargs)
